#! /usr/bin/env python

from lightflowPM import *
from math import *
import sys

s = scene()

s.lightOn( s.newLight( "soft", [ "position", vector3( 5.0, -5.0, 4.0 ), "color", vector3( 300.0, 300.0, 300.0 ) ] ) )


plastic = s.newMaterial( "standard", [ "ka", vector3( 0.2, 0.3, 0.4 ), "kc", vector3( 0.5, 0.8, 1.0 ), "kd", 0.5, "km", 0.1 ] )


s.materialBegin( plastic )

p = vector3array()

N = 50
M = 10
R = 1.0
r = 0.2

#alloc arrays
p.setSize( N * M )

for i in range( 0, N ) :
    u = 2.0 * pi * i / N

    for j in range( 0, M ) :
	v = 2.0 * pi * j / M

        index = i + j*N

	p.set( index, vector3( cos(u) * (R + cos(v)*r * (1.0 + cos(u*8.0)*0.8)),
			       sin(u) * (R + cos(v)*r * (1.0 + cos(u*8.0)*0.8)),
			       sin(v)*r * (1.0 + cos(u*8.0)*0.8) ) )

    if not (i % 10) :
        sys.stdout.write("\rallocating vertices %d %%" % ((i * 100) / N) )
        sys.stdout.flush()

print

s.addObject( s.newObject( "bspline",
			  [ "size", N, M,
			    "wrap", 1, 1,
			    "step", BSplineStep(), BSplineStep(),
			    "basis", BSplineBasis(), BSplineBasis(),
			    "points", p,
			    "tolerance", 0.02, 0.1, 0.05 ] ) )

# destroy arrays
del p

s.materialEnd()


saver = s.newImager( "tga-saver", [ "file", "bspline1.tga" ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole", [ "eye", vector3( -2, -3, 3 ), "aim", vector3( 0, 0, 0 ) ] )

s.imagerEnd()

s.render( camera, 400, 400 )
