#! /usr/bin/env python

from lightflowPM import *
from math import *

s = scene()

s.newInterface( "default",
		[ "trace-depth", 6 ] )

s.lightOn( s.newLight( "point",
		       [ "position", vector3( 300.0, -500.0, 300.0 ),
			 "color", vector3( 5.0, 5.0, 6.0 ),
			 "decay", 0.0 ] ) )


woodpattern = s.newPattern( "oldwood",
			    [ "vein-color", vector3( 0.8, 0.5, 0.3 ),
			      "darkening-amount", 0.2,
			      "node-amount", 0.2,
			      "scale", 0.5 ] )

wood = s.newMaterial( "standard",
		      [ "kc", vector3( 1.0, 0.7, 0.5 ),
			"kd", 0.7,
			"km", 0.2,
			"kc", woodpattern,
			"displacement", woodpattern ] )

metal = s.newMaterial( "standard",
		       [ "kc", vector3( 1.0, 0.9, 0.8 ),
			 "kr", vector3( 0.6, 0.6, 0.6 ),
			 "kd", 0.7,
			 "km", 0.05,
			 "displacement", woodpattern ] )

s.transformBegin( transform().rotationAroundZ( -pi / 4.0 ) )
s.transformBegin( transform().scaling( vector3( 2, 0.2, 0.2 ) ) )

waterbumps = s.newPattern( "multifractal",
			   [ "basis", "sin",
			     "scale", 2.0,
			     "depth", 0.05,
			     "turbulence.distortion", 1.0,
			     "turbulence.omega", 0.1, 0.3,
			     "turbulence.lambda", 2.0,
			     "turbulence.octaves", 5 ] )

s.transformEnd()
s.transformEnd()

water = s.newMaterial( "physical",
		       [ "fresnel", 1, 0.3, 0.0,
			 "IOR", 1.33,
			 "kr", vector3( 1, 1, 1 ),
			 "kt", vector3( 1, 1, 1 ),
			 "kd", 0.0,
			 "km", 0.03,
			 "shinyness", 0.5,
			 "displacement", waterbumps ] )

earthbumps = s.newPattern( "multifractal",
			   [ "basis", "sin",
			     "scale", 10.0,
			     "depth", 0.5,
			     "turbulence.distortion", 1.0,
			     "turbulence.omega", 0.1, 0.3,
			     "turbulence.lambda", 2.0,
			     "turbulence.octaves", 5 ] )

earth = s.newMaterial( "diffuse",
		       [ "kr", vector3( 0.8, 0.75, 0.7 ),
			 "displacement", earthbumps ] )


skyfog = s.newInterior( "fog",
			[ "kc", vector3( 0.35, 0.4, 0.7 ),
			  "kaf", 0.0007,
			  "density", 1.0 ] )

s.interiorBegin( skyfog )

sky = s.newMaterial( "matte", 
    [ "kc", vector3( 0.4, 0.4, 0.7 ),
      "shadowing", 0.0 ] )

s.interiorEnd()


glass = s.newMaterial( "generic",
		       [ "fresnel", 1, 0.0, 0.0,
			 "IOR", 1.65,
			 "kdr", vector3( 0, 0, 0 ),
 			 "kdt", vector3( 0, 0, 0 ),
			 "ksr", vector3( 1, 1, 1 ), vector3( 0.5, 0.8, 1 ),
			 "kst", vector3( 1, 1, 1 ), vector3( 1, 0.6, 0.2 ),
			 "kr", vector3( 1, 1, 1 ),
			 "kt", vector3( 1, 1, 1 ),
			 "km", 0.02,
			 "shinyness", 1.0,
			 "transmission", 1,
			 "shadowing", 0.0,
			 "radiosity", 0,
			 "caustics", 0, 0 ] )



s.materialBegin( wood )

for i in range(0, 75) :
    u = i / 74.0

    s.addObject( s.newObject( "box",
			      [ "position",
				vector3( u*18.0 - 0.1 - 15.0, -0.7, -0.02 ),
				vector3( u*18.0 + 0.1 - 15.0,  0.7,  0.00 ) ] ) )

s.addObject( s.newObject( "box",
			  [ "position",
			    vector3( -15.0, -0.65, -0.15 ),
			    vector3(   3.0, -0.55, -0.02 ) ] ) )

s.addObject( s.newObject( "box",
			  [ "position",
			    vector3( -15.0,  0.55, -0.15 ),
			    vector3(   3.0,  0.65, -0.02 ) ] ) )

for i in range(0, 5) :
    u = i / 4.0

    s.transformBegin( transform().translation( vector3( u*17.0 - 14.5, -0.45, -2.0 ) ) )

    s.addObject( s.newObject( "cylinder",
			      [ "radius", 0.1,
				"height", 2.0 ] ) )

    s.transformEnd()

    s.transformBegin( transform().translation( vector3( u*17.0 - 14.5, 0.45, -2.0 ) ) )

    s.addObject( s.newObject( "cylinder",
			      [ "radius", 0.1,
				"height", 2.0 ] ) )

    s.transformEnd()

s.materialEnd()

s.materialBegin( metal )

for i in range(0, 5) :
    u = (i+0.5) / 5.0

    s.transformBegin( transform().translation( vector3( u*17.0 - 14.5, 0.45, 0.0 ) ) )

    s.addObject( s.newObject( "cylinder",
			      [ "radius", 0.04,
				"height", 2.5 ] ) )

    s.transformBegin( transform().translation( vector3( 0, -0.3, 2.5 ) ) )
    s.transformBegin( transform().rotationAroundY( pi / 2.0 ) )

    torus = s.newObject( "torus",
			 [ "radii", 0.3, 0.04,
			   "region", 0.25, 0.0, 0.75, 1.0 ] )

    s.addObject( s.newObject( "surface-engine",
			      [ "surfaces", torus,
				"tolerance", 0.02, 0.1, 0.0 ] ) )

    s.transformEnd()
    s.transformEnd()

    s.transformBegin( transform().translation( vector3( 0, -0.6, 2.4 ) ) )

    s.addObject( s.newObject( "cone",
			      [ "radius", 0.25,
				"height", 0.15 ] ) )

    s.transformEnd()

    s.materialBegin( glass )
    s.transformBegin( transform().translation( vector3( 0, -0.6, 2.4 ) ) )
    s.transformBegin( transform().scaling( vector3( 1, 1, 2.0 ) ) )

    s.addObject( s.newObject( "sphere",
			      [ "radius", 0.15,
				"region", 0.0, 0.0, 1.0, 0.5 ] ) )

    s.addObject( s.newObject( "sphere",
			      [ "radius", 0.14,
				"region", 0.0, 0.0, 1.0, 0.5 ] ) )

    s.transformEnd()
    s.transformEnd()
    s.materialEnd()

    s.transformEnd()


s.materialEnd()



s.transformBegin( transform().translation( vector3( 0, 0, -1 ) ) )

s.materialBegin( water )

s.addObject( s.newObject( "disc", [ "radius", 1000.0 ] ) )

s.materialEnd()

s.transformEnd()


s.transformBegin( transform().scaling( vector3( 35, 35, 35 ) ) )

moonbumps = s.newPattern( "moon", [] )

s.transformEnd()

moon = s.newMaterial( "generic",
    [ "ka", vector3( 0.9 * 0.55, 0.9 * 0.55, 1.0 * 0.55 ),
      "ka", moonbumps,
      "kdr", vector3( 0.9 * 0.25, 0.9 * 0.25, 1.0 * 0.25 ),
      "kdr", moonbumps,
      "kdt", vector3( 0.9 * 0.4, 0.9 * 0.4, 1.0 * 0.4 ),
      "displacement", moonbumps,
      "shadowing", 0.0 ] )

mooninterior = s.newInterior( "halo",
    [ "kc", vector3( 0.8, 0.8, 1.0 ),
      "ke", 2.0,
      "distance", 100.0 ] )

s.interiorBegin( mooninterior )

moonhalo = s.newMaterial( "transparent", [] )

s.interiorEnd()

s.transformBegin( transform().translation( vector3( 400, 300, 37 ) ) )
s.transformBegin( transform().rotationAroundZ( -pi / 5.0 ) )
s.transformBegin( transform().rotationAroundX( pi / 2.2 ) )

s.materialBegin( moon )

s.addObject( s.newObject( "sphere", [ "radius", 35.0 ] ) )

s.materialEnd()

s.transformEnd()
s.transformEnd()

s.materialBegin( moonhalo )

s.addObject( s.newObject( "sphere", [ "radius", 49.0 ] ) )

s.materialEnd()

s.transformEnd()


s.transformBegin( transform().scaling( vector3( 1, 1, 0.1 ) ) )

s.materialBegin( sky )

s.addObject( s.newObject( "sphere", [ "radius", 1000.0 ] ) )

s.materialEnd()

s.transformEnd()


smooth = s.newImager( "smooth", [ "smooth", 0.65 ] )
saver  = s.newImager( "tga-saver", [ "file", "evening.tga", "imager", smooth ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole",
		      [ "eye", vector3( -4, -5, 2 ),
			"aim", vector3( 1, 0, 0 ) ] )

s.imagerEnd()


s.render( camera, 400, 570 )
