#! /usr/bin/env python

from math import *
from lightflowPM import *

s = scene()

s.lightOn( s.newLight( "soft",
		       [ "position", vector3( -4, -7, 6 ),
			 "color", vector3( 500, 500, 500 ) ] ) )

s.lightOn( s.newLight( "soft",
		       [ "position", vector3( 9, -5, 2 ),
			 "color", vector3( 350, 0.5 * 350, 0.6 * 350 ) ] ) )


threads = s.newPattern( "threads",
			[ "frequency", 6.0,
			  "dampzone", 0.2,
			  "depth", 0.1 ] )

geomview1 = s.newMaterial( "geom-view", [] )
geomview2 = s.newMaterial( "geom-view", ["displacement", threads] )


# model a mechanic die
s.materialBegin( geomview1 )

surfaces = intarray()

for i in range(0, 6) :
    u1 = 2.0 * 3.141592 * (i - 0.5) / 6.0
    u2 = 2.0 * 3.141592 * (i + 0.5) / 6.0

    sin_u1 = sin( u1 )
    sin_u2 = sin( u2 )
    cos_u1 = cos( u1 )
    cos_u2 = cos( u2 )

    surfaces.add( s.newObject( "patch",
			       [ "points",
				 vector3( cos_u1, sin_u1, -0.4 ), vector3( cos_u1, sin_u1, 0.4 ),
				 vector3( cos_u2, sin_u2, -0.4 ), vector3( cos_u2, sin_u2, 0.4 ) ] ) )

    surfaces.add( s.newObject( "triangle",
			       [ "points",
				 vector3( cos_u1, sin_u1, -0.4 ), vector3( cos_u2, sin_u2, -0.4 ), vector3( 0, 0, -0.4 ) ] ) )

    surfaces.add( s.newObject( "triangle",
			       [ "points",
				 vector3( cos_u1, sin_u1, 0.4 ), vector3( cos_u2, sin_u2, 0.4 ), vector3( 0, 0, 0.4 ) ] ) )

boundary = s.newObject( "boundary", [ "surfaces", surfaces ] )

s.transformBegin( transform().scaling( vector3( 1.0, 1.0, 0.4 ) ) )

sphere = s.newObject( "sphere", [ "radius", 1.3 ] )

s.transformEnd()

body = s.newObject( "solid",
		    [ "type", "intersection",
		      "solids", boundary, sphere ] )

s.transformBegin( transform().translation( vector3( 0.0, 0.0, -0.41 ) ) )

s.materialBegin( geomview2 )

cyl = s.newObject( "cylinder",
		   [ "radius", 0.5,
		     "height", 0.82 ] )

s.materialEnd()

disc1 = s.newObject( "disc", [ "radius", 0.5 ] )

s.transformEnd()

s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.41 ) ) )

disc2 = s.newObject( "disc", [ "radius", 0.5 ] )

s.transformEnd()

hole = s.newObject( "boundary", [ "surfaces", cyl, disc1, disc2 ] )

s.addObject( s.newObject( "solid",
			  [ "type", "difference",
			    "solids", body, hole ] ) )

s.materialEnd()



saver = s.newImager( "tga-saver", [ "file", "geomview1.tga" ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole",
    [ "eye", vector3( 2.1, -2.0,  1.8 ),
      "aim", vector3( 0.0,  0.0,  0.0 ) ] )

s.imagerEnd()

s.render( camera, 400, 400 )
