#! /usr/bin/env python

from lightflowPM import *
from math import *

s = scene()

s.newInterface( "default",
		[ "trace-depth", 6,
		  "radiosity-depth", 6, 6,
		  "radiosity-sampling", "default",
		  "radiosity-samples", 350,
		  "radiosity-threshold", 0.1,
		  "radiosity-reuse-distance", 0.8, 0.01,
		  "photon-count", 300000,
		  "photon-clustering-count", 1000, 75 ] )


#s.lightOn( s.newLight( "soft", [ "position", vector3( -6.0, -4.0, 6.0 ), "color", vector3( 150.0, 150.0, 150.0 ) ] ) )
#s.lightOn( s.newLight( "soft", [ "position", vector3(  3.0,  5.0, 8.0 ), "color", vector3( 75.0, 0.0, 0.0 ) ] ) )

s.lightOn( s.newLight( "point", [ "position", vector3( -6.0, -4.0, 6.0 ), "color", vector3( 150.0, 150.0, 150.0 ), "shadows", 1 ] ) )
s.lightOn( s.newLight( "point", [ "position", vector3(  3.0,  5.0, 8.0 ), "color", vector3( 75.0, 0.0, 0.0 ), "shadows", 1 ] ) )


whitewash = s.newMaterial( "diffuse",
			   [ "kr", vector3( 0.8, 0.8, 0.8 ) ] )

marblepattern = s.newPattern( "granite",
		     [ "color",
		       0.2, vector3( 0.8, 0.8, 0.8 ), vector3( 0.8, 0.8, 0.8 ),
		       0.5, vector3( 1.0, 1.0, 1.0 ), vector3( 1.0, 1.0, 1.0 ),
		       "value",
		       0.2, 1.00, 1.00,
		       0.5, 0.65, 0.65,
		       "scale", 0.8 ] )

marblepattern = s.newPattern( "granite",
		     [ "color",
		       0.2, vector3( 1.0, 1.0, 1.0 ), vector3( 1.0, 1.0, 1.0 ),
		       0.8, vector3( 0.8, 0.8, 0.8 ), vector3( 0.8, 0.8, 0.8 ),
		       "value",
		       0.2, 0.8, 0.8,
		       0.8, 1.0, 1.0,
		       "scale", 0.8 ] )

marble = s.newMaterial( "physical",
		       [ "fresnel", 1,
			 "IOR", 5.0,
			 "kr", vector3( 1.0, 1.0, 1.0 ),
			 "kr", marblepattern,
			 "kd", 1.00,
			 "kd", marblepattern,
			 "km", 0.05,
			 "shinyness", 1.0,
			 "radiosity", 1,
			 "caustics", 2, 2 ] )


starpattern = s.newPattern( "stars",
			    [ "color", vector3( 1.0, 0.85, 0.6 ),
			      "value", 0.75,
			      "scale", 0.1,
			      "coverage", 0.5,
			      "irregularity", 0.5 ] )

writingpattern = s.newPattern( "map",
			       [ "texture", s.newTexture( "Textures/hnytxt.tga" ) ] )

paperpattern = s.newPattern( "compose",
			     [ "patterns", starpattern, writingpattern ] )

paper = s.newMaterial( "physical",
		       [ "fresnel", 1,
			 "IOR", 7.0,
			 "ka", vector3( 0.1, 0.1, 0.1 ),
			 "kr", vector3( 0.9, 1.0, 1.0 ),
			 "kr", paperpattern,
			 "kd", 0.98,
			 "kd", starpattern,
			 "km", 0.05,
			 "shinyness", 1.0,
			 "radiosity", 1,
			 "caustics", 2, 2 ] )

gold = s.newMaterial( "physical",
		      [ "fresnel", 1,
			"IOR", 7.0,
			"kr", vector3( 1.0, 0.9, 0.4 ),
			"kd", 0.10,
			"km", 0.05,
			"shinyness", 1.0,
			"radiosity", 0,
			"caustics", 2, 2 ] )

bluemetal = s.newMaterial( "physical",
			[ "fresnel", 1,
			  "IOR", 7.0,
			  "kr", vector3( 0.2, 0.4, 1.0 ),
			  "kd", 0.10,
			  "km", 0.05,
			  "shinyness", 0.95,
			  "radiosity", 0,
			  "caustics", 2, 2 ] )

redmetal = s.newMaterial( "physical",
			[ "fresnel", 1,
			  "IOR", 7.0,
			  "kr", vector3( 1.0, 0.1, 0.07 ),
			  "kd", 0.10,
			  "km", 0.05,
			  "shinyness", 1.0,
			  "radiosity", 0,
			  "caustics", 2, 2 ] )

glass = s.newMaterial( "generic",
		       [ "fresnel", 1,
			 "IOR", 1.57,
			 "kdr", vector3( 0, 0, 0 ),
 			 "kdt", vector3( 0, 0, 0 ),
			 "ksr", vector3( 1, 1, 1 ), vector3( 0.5, 0.8, 1 ),
			 "kst", vector3( 1, 1, 1 ), vector3( 0.5, 0.8, 1 ),
			 "kr", vector3( 1, 1, 1 ),
			 "kt", vector3( 1, 1, 1 ),
			 "km", 0.08,
			 "shinyness", 1.0,
			 "transmission", 0,
			 "radiosity", 0,
			 "caustics", 2, 2 ] )


s.materialBegin( whitewash )

s.addObject( s.newObject( "box", [ "position", vector3( -10, -10, -1.0 ), vector3( 10, 10, 10 ) ] ) )

s.materialEnd()

s.materialBegin( marble )

s.addObject( s.newObject( "patch",
			  [ "points",
			    vector3( -10, -10, 0.0 ), vector3( 10, -10, 0.0 ),
			    vector3( -10,  10, 0.0 ), vector3( 10,  10, 0.0 ) ] ) )

s.materialEnd()



s.materialBegin( bluemetal )

s.transformBegin( transform().translation( vector3( 0.7, 1.2, 0.7 ) ) )
s.transformBegin( transform().rotationAroundZ( -3.141592 / 4.0 ) )
s.transformBegin( transform().rotationAroundX( 3.141592 / 6.0 ) )

s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.6 ) ) )

s.addObject( s.newObject( "cone",
			  [ "radius", 0.25,
			    "height", 0.60,
			    "region", 0.0, 1.0, 0.4 ] ) )

s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.6 * 0.4 ) ) )

s.addObject( s.newObject( "disc", [ "radius", 0.25 * (1.0 - 0.4) ] ) )

s.materialBegin( glass )
s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.03 ) ) )
s.addObject( s.newObject( "sphere", [ "radius", 0.06 ] ) )
s.transformEnd()
s.materialEnd()

s.transformEnd()

s.transformEnd()

s.materialBegin( gold )

s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.6 + 0.6*0.4 ) ) )
s.transformBegin( transform().rotationAroundX( 3.141592 / 4.0 ) )
s.transformBegin( transform().translation( vector3( 0.0, 0.15 + 0.015, 0.0 ) ) )

torus = s.newObject( "torus", [ "radii", 0.15, 0.015 ] )

s.addObject( s.newObject( "surface-engine",
			  [ "surfaces", torus,
			    "tolerance", 0.02, 0.1, 0.0 ] ) )

s.transformEnd()
s.transformEnd()
s.transformEnd()

s.materialEnd()
	     
s.addObject( s.newObject( "sphere", [ "radius", 0.7 ] ) )

s.transformEnd()
s.transformEnd()
s.transformEnd()

s.materialEnd()



s.materialBegin( redmetal )

s.transformBegin( transform().translation( vector3( -0.8, 0.55, 0.7 ) ) )
s.transformBegin( transform().rotationAroundZ( 3.141592 / 5.0 ) )
s.transformBegin( transform().rotationAroundX( 3.141592 / 8.0 ) )

s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.6 ) ) )

s.addObject( s.newObject( "cone",
			  [ "radius", 0.25,
			    "height", 0.60,
			    "region", 0.0, 1.0, 0.4 ] ) )

s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.6 * 0.4 ) ) )

s.addObject( s.newObject( "disc", [ "radius", 0.25 * (1.0 - 0.4) ] ) )

s.materialBegin( glass )
s.transformBegin( transform().translation( vector3( 0.0, 0.0, 0.03 ) ) )
s.addObject( s.newObject( "sphere", [ "radius", 0.06 ] ) )
s.transformEnd()
s.materialEnd()

s.transformEnd()

s.transformEnd()

s.materialBegin( gold )

s.transformBegin( transform().translation( vector3( 0.0, 0, 0.6 + 0.6*0.4 ) ) )
s.transformBegin( transform().rotationAroundX( -3.141592 / 3.0 ) )
s.transformBegin( transform().translation( vector3( 0.0, -0.15 - 0.015, 0.0 ) ) )

torus = s.newObject( "torus", [ "radii", 0.15, 0.015 ] )

s.addObject( s.newObject( "surface-engine",
			  [ "surfaces", torus,
			    "tolerance", 0.02, 0.1, 0.0 ] ) )

s.transformEnd()
s.transformEnd()
s.transformEnd()

s.materialEnd()

s.addObject( s.newObject( "sphere", [ "radius", 0.7 ] ) )

s.transformEnd()
s.transformEnd()
s.transformEnd()

s.materialEnd()



s.materialBegin( paper )

s.transformBegin( transform().rotationAroundZ( 3.141592 / 6 ) )

s.addObject( s.newObject( "patch",
			  [ "points",
			    vector3( -1,  0.7, 0.01 ), vector3( 1,  0.7, 0.01 ),
			    vector3( -1, -0.7, 0.01 ), vector3( 1, -0.7, 0.01 ) ] ) )

s.transformEnd()

s.materialEnd()


film  = s.newImager( "film", [ "grain", 0.0 ] )
saver = s.newImager( "tga-saver", [ "file", "hny.tga", "imager", film ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole",
		      [ "eye", vector3( 0, -2.5, 2.5 ),
			"aim", vector3( 0, 1.0, 0 ),
			"fov", atan( 0.5 / 0.9 )*2.0 ] )

s.imagerEnd()


s.radiosity()

s.render( camera, 400, 570 )
