#! /usr/bin/env python

# This example has been inspired by a tutorial for Radiance that can be found
# in the book "Rendering with Radiance", published by Morgan Kaufmann.

from lightflowPM import *
from math import *

s = scene()

s.newInterface( "default",
		[ "trace-depth", 6,
		  "radiosity-file", "radiance.rad", "load", # use precomputed radiosity
		  "radiosity-depth", 6, 6,
		  "radiosity-sampling", "default",
		  "radiosity-samples", 400,
		  "radiosity-threshold", 0.1,
		  "radiosity-reuse-distance", 0.4, 0.01,
		  "photon-count", 300000,
		  "photon-clustering-count", 1000, 75 ] )


s.lightOn( s.newLight( "conic",
		       [ "position", vector3( 1.0, -1.0, 2.48 ),
			 "direction", vector3( 0, 0, -1 ),
			 "angle", 0.0, 3.141592 / 2.0,
			 "color", vector3( 8, 8, 8 ) ] ) )

s.lightOn( s.newLight( "soft-conic",
		       [ "position", vector3( 0.75, -1.48, 1.25 ),
			 "direction", vector3( 0, 1, 0 ),
			 "angle", 0.0, 3.141592 / 2.0,
			 "radius", 0.02,
			 "samples", 7,
			 "color", vector3( 7, 7, 8 ),
			 "file", "radiance.shd", "load" ] ) ) # use precomputed shadow-maps



neon = s.newMaterial( "matte", [ "kc", vector3( 6, 6, 7 ), "shadowing", 0.0 ] )

whitewash = s.newMaterial( "diffuse",
                           [ "kr", vector3( 0.85, 0.85, 0.85 ) ] )

redwash = s.newMaterial( "generic",
                         [ "kdr", vector3( 0.46 * 1.3, 0.32 * 1.3, 0.3 * 1.3 ),
                           "ksr", vector3( 0.5, 0.5, 0.5 ),
                           "km", 0.07,
                           "shinyness", 1.0,
                           "radiosity", 1,
                           "caustics", 0, 0] )

bluewash = s.newMaterial( "generic",
                          [ "kdr", vector3( 0.3, 0.4, 0.8 ),
                            "ksr", vector3( 0.5, 0.5, 0.5 ),
                            "km", 0.07,
                            "shinyness", 1.0,
                            "radiosity", 1,
                            "caustics", 0, 0 ] )

trnswash = s.newMaterial( "generic",
                          [ "kdr", vector3( 0.8, 0.8, 0.8 ),
                            "ksr", vector3( 0.5, 0.5, 0.5 ),
                            "km", 0.07,
                            "shinyness", 1.0,
                            "radiosity", 1,
                            "caustics", 0, 0,
                            "visibility", 1 ] )

glass = s.newMaterial( "generic",
		       [ "fresnel", 1,
			 "IOR", 1.57,
			 "kdr", vector3( 0, 0, 0 ),
 			 "kdt", vector3( 0, 0, 0 ),
			 "ksr", vector3( 1, 1, 1 ),
			 "kst", vector3( 1, 1, 1 ),
			 "kr", vector3( 1, 1, 1 ),
			 "kt", vector3( 1, 1, 1 ),
			 "km", 0.02,
			 "shinyness", 1.0,
			 "transmission", 0,
			 "radiosity", 0,
			 "caustics", 2, 2 ] )


s.materialBegin( neon )

s.addObject( s.newObject( "patch",
			  [ "points",
			    vector3( 0.8, -1.2, 2.49 ), vector3( 1.2, -1.2, 2.49 ),
			    vector3( 0.8, -0.8, 2.49 ), vector3( 1.2, -0.8, 2.49 ) ] ) )

s.addObject( s.newObject( "patch",
			  [ "points",
			    vector3( 0.4, -1.49, 1.0 ), vector3( 1.1, -1.49, 1.0 ),
			    vector3( 0.4, -1.49, 1.5 ), vector3( 1.1, -1.49, 1.5 ) ] ) )

s.materialEnd()



s.materialBegin( whitewash )

s.addObject( s.newObject( "box",
			  [ "position", vector3( -1.5, -1.5, -0.1 ), vector3( 1.5, 1.5, 2.5 ) ] ) )

s.materialEnd()

s.materialBegin( redwash )

s.addObject( s.newObject( "patch",
			  [ "points",
			    vector3( -1.5, -1.5, 0.0 ), vector3( 1.5, -1.5, 0.0 ),
			    vector3( -1.5,  1.5, 0.0 ), vector3( 1.5,  1.5, 0.0 ) ] ) )

s.materialEnd()

s.transformBegin( transform().translation( vector3( 0.75, 0.9, 0.0 ) ) )
s.transformBegin( transform().rotationAroundZ( pi / 10 ) )

s.materialBegin( bluewash )

s.addObject( s.newObject( "box",
			  [ "position", vector3( -0.25, -0.25, 0.0 ), vector3( 0.25, 0.25, 0.5 ) ] ) )

s.materialEnd()

s.transformBegin( transform().translation( vector3( 0.1, 0, 0.63 ) ) )

s.materialBegin( glass )


s.addObject( s.newObject( "sphere", [ "radius", 0.13 ] ) )

s.materialEnd()

s.transformEnd()

s.transformEnd()
s.transformEnd()



dof   = s.newImager( "dof", [ "radius", 0.01, "depth", -1.0, "mask", 20 ] )
saver = s.newImager( "tga-saver", [ "file", "radiance.tga", "imager", dof ] )

s.imagerBegin( saver )

camera = s.newCamera( "pinhole",
		      [ "eye", vector3( 0.00, -1.45, 1.38 ),
			"aim", vector3( 0.85,  0.90, 0.30 ),
			"fov", pi/6.0 ] )

s.imagerEnd()


s.radiosity()

s.render( camera, 300, 300 )
