// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfIOStream.h                                                                 //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Jacopo Pantaleoni. All rights reserved.           //
// ============================================================================ //

#ifndef _LfIOStream_H
#define _LfIOStream_H 1

#include <Lightflow/LfError.h>
#include <iostream.h>
#include <iomanip.h>


// Abstract Input ASCII Stream
class LfAPI_CS LfIAStream
{
public:
	LfIAStream(void) {}
	virtual ~LfIAStream() {}

	virtual LfBool HasSeek(void) const { return LfFalse; }
	virtual void   Seek(long nbytes) {}
	virtual long   Tell(void) { return 0; }

	virtual void Read (char&)           = 0;
	virtual void Read (unsigned char&)  = 0;
	virtual void Read (short&)          = 0;
	virtual void Read (unsigned short&) = 0;
	virtual void Read (int&)            = 0;
	virtual void Read (unsigned int&)   = 0;
	virtual void Read (long&)           = 0;
	virtual void Read (unsigned long&)  = 0;
	virtual void Read (float&)          = 0;
	virtual void Read (double&)         = 0;
	virtual void Read (char*)           = 0;
};
// Abstract Output ASCII Stream
class LfAPI_CS LfOAStream
{
public:
	LfOAStream(void) {}
	virtual ~LfOAStream() {}

	virtual LfBool HasSeek(void) const { return LfFalse; }
	virtual void   Seek(long nbytes) {}
	virtual long   Tell(void) { return 0; }

	virtual void Write (char)           = 0;
	virtual void Write (unsigned char)  = 0;
	virtual void Write (short)          = 0;
	virtual void Write (unsigned short) = 0;
	virtual void Write (int)            = 0;
	virtual void Write (unsigned int)   = 0;
	virtual void Write (long)           = 0;
	virtual void Write (unsigned long)  = 0;
	virtual void Write (float)          = 0;
	virtual void Write (double)         = 0;
	virtual void Write (const char*)    = 0;
};

// Abstract Input Binary Stream
class LfAPI_CS LfIBStream
{
public:
	LfIBStream(void) {}
	virtual ~LfIBStream() {}

	virtual LfBool HasSeek(void) const { return LfFalse; }
	virtual void   Seek(long nbytes) {}
	virtual long   Tell(void) { return 0; }

	virtual void Read(int nbytes, void* data) = 0;
};
// Abstract Output Binary Stream
class LfAPI_CS LfOBStream
{
public:
	LfOBStream(void) {}
	virtual ~LfOBStream() {}

	virtual LfBool HasSeek(void) const { return LfFalse; }
	virtual void   Seek(long nbytes) {}
	virtual long   Tell(void) { return 0; }

	virtual void Write(int nbytes, const void* data) = 0;
};



inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, char           v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned char  v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, short          v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned short v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, int            v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned int   v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, long           v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned long  v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, float          v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, double         v);
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, const char*    v);

inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, char&           v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned char&  v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, short&          v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned short& v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, int&            v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned int&   v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, long&           v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned long&  v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, float&          v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, double&         v);
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, char*           v);

inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, char           v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned char  v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, short          v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned short v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, int            v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned int   v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, long           v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned long  v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, float          v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, double         v);
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, const char*    v);

inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, char&           v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned char&  v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, short&          v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned short& v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, int&            v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned int&   v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, long&           v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned long&  v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, float&          v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, double&         v);
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, char*           v);



inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, char v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned char v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, short v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned short v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, int v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned int v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, long v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, unsigned long v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, float v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, double v)
{
	s.Write( v );
	return s;
}
inline LfAPI_CS LfOAStream& operator<< ( LfOAStream& s, const char* v)
{
	s.Write( v );
	return s;
}



inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, char& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned char& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, short& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned short& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, int& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned int& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, long& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, unsigned long& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, float& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, double& v)
{
	s.Read( v );
	return s;
}
inline LfAPI_CS LfIAStream& operator>> ( LfIAStream& s, char* v)
{
	s.Read( v );
	return s;
}



inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, char v)
{
	s.Write( sizeof( char ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned char v)
{
	s.Write( sizeof( unsigned char ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, short v)
{
	s.Write( sizeof( short ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned short v)
{
	s.Write( sizeof( unsigned short ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, int v)
{
	s.Write( sizeof( int ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned int v)
{
	s.Write( sizeof( unsigned int ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, long v)
{
	s.Write( sizeof( long ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, unsigned long v)
{
	s.Write( sizeof( unsigned long ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, float v)
{
	s.Write( sizeof( float ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, double v)
{
	s.Write( sizeof( double ), (void*)&v );
	return s;
}
inline LfAPI_CS LfOBStream& operator<< ( LfOBStream& s, const char* v)
{
	LfInt length = strlen( v );

	s.Write( sizeof( LfInt ), (void*)&length );
	s.Write( length, (void*)v );
	return s;
}



inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, char& v)
{
	s.Read( sizeof( char ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned char& v)
{
	s.Read( sizeof( unsigned char ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, short& v)
{
	s.Read( sizeof( short ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned short& v)
{
	s.Read( sizeof( unsigned short ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, int& v)
{
	s.Read( sizeof( int ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned int& v)
{
	s.Read( sizeof( unsigned int ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, long& v)
{
	s.Read( sizeof( long ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, unsigned long& v)
{
	s.Read( sizeof( unsigned long ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, float& v)
{
	s.Read( sizeof( float ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, double& v)
{
	s.Read( sizeof( double ), (void*)&v );
	return s;
}
inline LfAPI_CS LfIBStream& operator>> ( LfIBStream& s, char* v)
{
	LfInt length;

	s.Read( sizeof( LfInt ), (void*)&length );
	s.Read( length, (void*)v );
	v[ length ] = '\0';
	return s;
}


#endif
