// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfMatrix2.h                                                                  //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Marco and Jacopo Pantaleoni. All rights reserved. //
// ============================================================================ //

#ifndef _LfMatrix2_h
#define _LfMatrix2_h 1

#include <Lightflow/LfVector2.h>


class LfAPI_CS LfMatrix2
{
private:
	LfVector2 r[2];

public:
inline                      LfMatrix2     ();
inline                      LfMatrix2     (const LfMatrix2&);
inline                      LfMatrix2     (const LfVector2&,  const LfVector2&);
inline                      LfMatrix2     (const float  v[2][2]);
inline                      LfMatrix2     (const double v[2][2]);

inline        LfMatrix2&    operator  = (const LfMatrix2&);
inline        LfMatrix2&    operator += (const LfMatrix2&);
inline        LfMatrix2&    operator -= (const LfMatrix2&);
inline        LfMatrix2&    operator *= (const LfMatrix2&);
inline        LfMatrix2&    operator *= (LfFloat);
inline        LfMatrix2&    operator /= (LfFloat);

inline        LfVector2&       operator [] (int);
inline        const LfVector2& Get(int) const;
inline        void             Set(int, const LfVector2&);

inline        LfFloat       Det() const;

friend inline LfAPI_CS int           operator == (const LfMatrix2&,  const LfMatrix2&);
friend inline LfAPI_CS int           operator != (const LfMatrix2&,  const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2     operator  - (const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2     operator  + (const LfMatrix2&,  const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2     operator  - (const LfMatrix2&,  const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2     operator  * (const LfMatrix2&,  const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2     operator  * (const LfMatrix2&,  LfFloat);
friend inline LfAPI_CS LfMatrix2     operator  * (LfFloat,           const LfMatrix2&);
friend inline LfAPI_CS LfVector2     operator  * (const LfVector2&,  const LfMatrix2&);
friend inline LfAPI_CS LfVector2     operator  * (const LfMatrix2&,  const LfVector2&);
friend inline LfAPI_CS LfMatrix2&    Multiply    (const LfMatrix2&,  const LfMatrix2&,  LfMatrix2&);
friend inline LfAPI_CS LfVector2&    Multiply    (const LfVector2&,  const LfMatrix2&,  LfVector2&);
friend inline LfAPI_CS LfVector2&    Multiply    (const LfMatrix2&,  const LfVector2&,  LfVector2&);
friend inline LfAPI_CS LfMatrix2     operator  / (const LfMatrix2&,  LfFloat);
friend inline LfAPI_CS LfMatrix2     Transpose   (const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2&    Transpose   (const LfMatrix2&,  LfMatrix2&);
/*friend inline LfAPI_CS LfMatrix2	Invert      (const LfMatrix2&);
friend inline LfAPI_CS LfMatrix2	    Invert      (const LfMatrix2&,  int&);*/
friend inline LfAPI_CS int           Invert      (const LfMatrix2&,  LfMatrix2&);
friend inline LfAPI_CS LfFloat       Det         (const LfMatrix2&);
};


//
// I M P L E M E N T A T I O N
//

#ifdef _V
#undef _V
#endif
#define _V(v, i)   (((LfVector2&)v).x[(i)])

#ifdef _M
#undef _M
#endif
#define _M(m, i, j)   (((LfMatrix2&)m).r[(i)][(j)])

#ifdef _CM
#undef _CM
#endif
#define _CM(m, i, j)  ((const LfFloat&)(((LfMatrix2&)m).r[(i)][(j)]))


//
// LfMatrix2 inline methods
//

inline LfMatrix2::LfMatrix2() { }

inline LfMatrix2::LfMatrix2(const LfMatrix2& m)
{
	r[0] = m.r[0];
	r[1] = m.r[1];
}

inline LfMatrix2::LfMatrix2(const LfVector2& r0, const LfVector2& r1)
{
	r[0] = r0;
	r[1] = r1;
}

inline LfMatrix2::LfMatrix2(const float  v[2][2])
{
	r[0][0] = v[0][0];
	r[0][1] = v[0][1];

	r[1][0] = v[1][0];
	r[1][1] = v[1][1];
}

inline LfMatrix2::LfMatrix2(const double v[2][2])
{
	r[0][0] = v[0][0];
	r[0][1] = v[0][1];

	r[1][0] = v[1][0];
	r[1][1] = v[1][1];
}

inline LfMatrix2& LfMatrix2::operator  = (const LfMatrix2& m)
{
	r[0] = m.r[0];
	r[1] = m.r[1];
	return *this;
}

inline LfMatrix2& LfMatrix2::operator += (const LfMatrix2& m)
{
	r[0] += m.r[0];
	r[1] += m.r[1];
	return *this;
}

inline LfMatrix2& LfMatrix2::operator -= (const LfMatrix2& m)
{
	r[0] -= m.r[0];
	r[1] -= m.r[1];
	return *this;
}

// SUBOPTIMAL
inline LfMatrix2& LfMatrix2::operator *= (const LfMatrix2& m)
// WARNING: This routine uses two temporary variables, so, if you need
//          to multiply two matrices two obtain a third matrix, AVOID
//          this method. Use instead:
//            LfMatrix2& Multiply(const LfMatrix2& m1, const LfMatrix2& m2, LfMatrix2& r)
{
	LfFloat tmp = r[0][0];

	r[0][0] = r[0][0]*_M(m,0,0) + r[0][1]*_M(m,1,0);
	r[0][1] = tmp    *_M(m,0,1) + r[0][1]*_M(m,1,1);


	tmp = r[1][0];

	r[1][0] = r[1][0]*_M(m,0,0) + r[1][1]*_M(m,1,0);
	r[1][1] = tmp    *_M(m,0,1) + r[1][1]*_M(m,1,1);
	return *this;
}

inline LfMatrix2& LfMatrix2::operator *= (LfFloat k)
{
	r[0] *= k;
	r[1] *= k;
	return *this;
}

inline LfMatrix2& LfMatrix2::operator /= (LfFloat k)
{
	r[0] /= k;
	r[1] /= k;
	return *this;
}

inline LfVector2& LfMatrix2::operator [] (int i)
{
	return r[i];
}

inline const LfVector2& LfMatrix2::Get(int i) const
{
	return r[i];
}
inline void LfMatrix2::Set(int i, const LfVector2& v)
{
	r[i] = v;
}

inline LfFloat LfMatrix2::Det() const
{
	return (_CM(*this,0,0)*_CM(*this,1,1) - _CM(*this,0,1)*_CM(*this,1,0));
}


//
// LfMatrix2 inline functions (not members)
//

inline LfAPI_CS int operator == (const LfMatrix2& a, const LfMatrix2& b)
{
	return ((a.r[0] == b.r[0]) && (a.r[1] == b.r[1]));
}

inline LfAPI_CS int operator != (const LfMatrix2& a, const LfMatrix2& b)
{
	return !(a == b);
}

inline LfAPI_CS LfMatrix2 operator  - (const LfMatrix2& a)
{
	return (LfMatrix2(a) *= -1.0);
}

inline LfAPI_CS LfMatrix2 operator  + (const LfMatrix2& a, const LfMatrix2& b)
{
	LfMatrix2 r;
	r.r[0][0] = _M(a,0,0) + _M(b,0,0);
	r.r[0][1] = _M(a,0,1) + _M(b,0,1);
	r.r[1][0] = _M(a,1,0) + _M(b,1,0);
	r.r[1][1] = _M(a,1,1) + _M(b,1,1);

	return r;
}

inline LfAPI_CS LfMatrix2 operator  - (const LfMatrix2& a, const LfMatrix2& b)
{
	LfMatrix2 r;
	r.r[0][0] = _M(a,0,0) - _M(b,0,0);
	r.r[0][1] = _M(a,0,1) - _M(b,0,1);
	r.r[1][0] = _M(a,1,0) - _M(b,1,0);
	r.r[1][1] = _M(a,1,1) - _M(b,1,1);

	return r;
}

inline LfAPI_CS LfMatrix2 operator  * (const LfMatrix2& a, const LfMatrix2& b)
{
	LfMatrix2 r;

	return Multiply(a, b, r);
}

inline LfAPI_CS LfMatrix2 operator  * (const LfMatrix2& a, LfFloat k)
{
	LfMatrix2 r;
	r.r[0][0] = _M(a,0,0) * k;
	r.r[0][1] = _M(a,0,1) * k;
	r.r[1][0] = _M(a,1,0) * k;
	r.r[1][1] = _M(a,1,1) * k;

	return r;
}

inline LfAPI_CS LfMatrix2 operator  * (LfFloat k, const LfMatrix2& a)
{
	return (a * k);
}

inline LfAPI_CS LfVector2 operator * (const LfVector2& v, const LfMatrix2& m)
{
	LfVector2 r;

	return Multiply(v, m, r);
}

inline LfAPI_CS LfVector2 operator * (const LfMatrix2& m, const LfVector2& v)
{
	LfVector2 r;

	return Multiply(m, v, r);
}

inline LfAPI_CS LfMatrix2& Multiply(const LfMatrix2& a, const LfMatrix2& b, LfMatrix2& r)
{
	r.r[0][0] = _M(a,0,0)*_M(b,0,0) + _M(a,0,1)*_M(b,1,0);
	r.r[0][1] = _M(a,0,0)*_M(b,0,1) + _M(a,0,1)*_M(b,1,1);
	r.r[1][0] = _M(a,1,0)*_M(b,0,0) + _M(a,1,1)*_M(b,1,0);
	r.r[1][1] = _M(a,1,0)*_M(b,0,1) + _M(a,1,1)*_M(b,1,1);

	return r;
}

// OPTIMAL
inline LfAPI_CS LfVector2& Multiply(const LfVector2& v, const LfMatrix2& m, LfVector2& r)
{
	r[0] = _V(v,0)*_M(m,0,0) + _V(v,1)*_M(m,1,0);
	r[1] = _V(v,0)*_M(m,0,1) + _V(v,1)*_M(m,1,1);

	return r;
}

// OPTIMAL
inline LfAPI_CS LfVector2& Multiply(const LfMatrix2& m, const LfVector2& v, LfVector2& r)
{
	r[0] = _M(m,0,0)*_V(v,0) + _M(m,0,1)*_V(v,1);
	r[1] = _M(m,1,0)*_V(v,0) + _M(m,1,1)*_V(v,1);

	return r;
}

inline LfAPI_CS LfMatrix2 operator  / (const LfMatrix2& a, LfFloat k)
{
	LfMatrix2 r;
	r.r[0][0] = _M(a,0,0) / k;
	r.r[0][1] = _M(a,0,1) / k;
	r.r[1][0] = _M(a,1,0) / k;
	r.r[1][1] = _M(a,1,1) / k;

	return r;
}

inline LfAPI_CS LfMatrix2 Transpose(const LfMatrix2& m)
{
	LfMatrix2 r;

	return Transpose(m, r);
}

inline LfAPI_CS LfMatrix2& Transpose(const LfMatrix2& m, LfMatrix2& r)
{
	r.r[0][0] = _M(m,0,0);
	r.r[0][1] = _M(m,1,0);
	r.r[1][0] = _M(m,0,1);
	r.r[1][1] = _M(m,1,1);

	return r;
}

inline LfAPI_CS int Invert(const LfMatrix2& a, LfMatrix2& r)
{
	LfFloat d = a.Det();
	if (__LfZero__(d)) return 0;

	d = 1/d;

	r.r[0][0] =  _M(a,1,1) * d;
	r.r[0][1] = -_M(a,0,1) * d;
	r.r[1][0] = -_M(a,1,0) * d;
	r.r[1][1] =  _M(a,0,0) * d;

	return 1;
}

inline LfAPI_CS LfFloat Det(const LfMatrix2& m)
{
	return m.Det();
}

#undef _M
#undef _CM

#endif /* _LfMatrix2_h */
