// ============================================================================ //
// Lightflow Rendering Interface                                                //
// ============================================================================ //
// LfMatrix4.h                                                                  //
// ============================================================================ //
//                                                                              //
// The contents of this file are subject to the Lightflow Rendering Interface   //
// License (the License); you may not use this file except in compliance with   //
// the License. You may obtain a copy of the License by sending an email at     //
// jp@lightflowtech.com with subject LRI-LICENSE.                               //
//                                                                              //
// ============================================================================ //
// Copyright (c) 1995-2000 by Marco and Jacopo Pantaleoni. All rights reserved. //
// ============================================================================ //

#ifndef _LfMatrix4_h
#define _LfMatrix4_h 1

#include <Lightflow/LfVector4.h>


class LfAPI_CS LfMatrix4
{
private:
	LfVector4 r[4];

public:
inline                      LfMatrix4   ();
inline                      LfMatrix4   (const LfMatrix4&);
inline                      LfMatrix4   (const LfVector4&, const LfVector4&, const LfVector4&, const LfVector4&);
inline                      LfMatrix4   (const float  v[4][4]);
inline                      LfMatrix4   (const double v[4][4]);

inline        LfMatrix4&    operator  = (const LfMatrix4&);
inline        LfMatrix4&    operator += (const LfMatrix4&);
inline        LfMatrix4&    operator -= (const LfMatrix4&);
inline        LfMatrix4&    operator *= (const LfMatrix4&);
inline        LfMatrix4&    operator *= (LfFloat);
inline        LfMatrix4&    operator /= (LfFloat);

inline        LfVector4&       operator [] (int);
inline        const LfVector4& Get(int) const;
inline        void             Set(int, const LfVector4&);

inline        LfFloat       Det() const;

friend inline LfAPI_CS int           operator == (const LfMatrix4&,  const LfMatrix4&);
friend inline LfAPI_CS int           operator != (const LfMatrix4&,  const LfMatrix4&);
friend inline LfAPI_CS LfMatrix4     operator  - (const LfMatrix4&);
friend inline LfAPI_CS LfMatrix4     operator  + (const LfMatrix4&,  const LfMatrix4&);
friend inline LfAPI_CS LfMatrix4     operator  - (const LfMatrix4&,  const LfMatrix4&);
friend inline LfAPI_CS LfMatrix4     operator  * (const LfMatrix4&,  const LfMatrix4&);
friend inline LfAPI_CS LfMatrix4     operator  * (const LfMatrix4&,  LfFloat);
friend inline LfAPI_CS LfMatrix4     operator  * (LfFloat,           const LfMatrix4&);
friend inline LfAPI_CS LfVector4     operator  * (const LfVector4&,  const LfMatrix4&);
friend inline LfAPI_CS LfVector4     operator  * (const LfMatrix4&,  const LfVector4&);
friend inline LfAPI_CS LfMatrix4&    Multiply    (const LfMatrix4&,  const LfMatrix4&,  LfMatrix4&);
friend inline LfAPI_CS LfVector4&    Multiply    (const LfVector4&,  const LfMatrix4&,  LfVector4&);
friend inline LfAPI_CS LfVector4&    Multiply    (const LfMatrix4&,  const LfVector4&,  LfVector4&);
friend inline LfAPI_CS LfMatrix4     operator  / (const LfMatrix4&,  LfFloat);
friend inline LfAPI_CS LfMatrix4     Transpose   (const LfMatrix4&);
friend inline LfAPI_CS LfMatrix4&    Transpose   (const LfMatrix4&,  LfMatrix4&);
/*friend inline LfAPI_CS LfMatrix4   Invert      (const LfMatrix4&);
  friend inline LfAPI_CS LfMatrix4   Invert      (const LfMatrix4&,  int&);*/
friend inline LfAPI_CS int           Invert      (const LfMatrix4&,  LfMatrix4&);
friend inline LfAPI_CS LfFloat       Det         (const LfMatrix4&);
};


//
// I M P L E M E N T A T I O N
//

#ifdef _V
#undef _V
#endif
#define _V(v, i)   (((LfVector4&)v).x[(i)])

#ifdef _M
#undef _M
#endif
#define _M(m, i, j)   (((LfMatrix4&)m).r[(i)][(j)])

#ifdef _CM
#undef _CM
#endif
#define _CM(m, i, j)  ((const LfFloat&)(((LfMatrix4&)m).r[(i)][(j)]))


//
// LfMatrix4 inline methods
//

inline LfMatrix4::LfMatrix4() { }

inline LfMatrix4::LfMatrix4(const LfMatrix4& m)
{
	r[0] = m.r[0];
	r[1] = m.r[1];
	r[2] = m.r[2];
	r[3] = m.r[3];
}

inline LfMatrix4::LfMatrix4(const LfVector4& r0, const LfVector4& r1, const LfVector4& r2, const LfVector4& r3)
{
	r[0] = r0;
	r[1] = r1;
	r[2] = r2;
	r[3] = r3;
}

inline LfMatrix4::LfMatrix4(const float  v[4][4])
{
	r[0][0] = v[0][0];
	r[0][1] = v[0][1];
	r[0][2] = v[0][2];
	r[0][3] = v[0][3];

	r[1][0] = v[1][0];
	r[1][1] = v[1][1];
	r[1][2] = v[1][2];
	r[1][3] = v[1][3];

	r[2][0] = v[2][0];
	r[2][1] = v[2][1];
	r[2][2] = v[2][2];
	r[2][3] = v[2][3];

	r[3][0] = v[3][0];
	r[3][1] = v[3][1];
	r[3][2] = v[3][2];
	r[3][3] = v[3][3];
}

inline LfMatrix4::LfMatrix4(const double v[4][4])
{
	r[0][0] = v[0][0];
	r[0][1] = v[0][1];
	r[0][2] = v[0][2];
	r[0][3] = v[0][3];

	r[1][0] = v[1][0];
	r[1][1] = v[1][1];
	r[1][2] = v[1][2];
	r[1][3] = v[1][3];

	r[2][0] = v[2][0];
	r[2][1] = v[2][1];
	r[2][2] = v[2][2];
	r[2][3] = v[2][3];

	r[3][0] = v[3][0];
	r[3][1] = v[3][1];
	r[3][2] = v[3][2];
	r[3][3] = v[3][3];
}

inline LfMatrix4& LfMatrix4::operator  = (const LfMatrix4& m)
{
	r[0] = m.r[0];
	r[1] = m.r[1];
	r[2] = m.r[2];
	r[3] = m.r[3];

	return *this;
}

inline LfMatrix4& LfMatrix4::operator += (const LfMatrix4& m)
{
	r[0] += m.r[0];
	r[1] += m.r[1];
	r[2] += m.r[2];
	r[3] += m.r[3];

	return *this;
}

inline LfMatrix4& LfMatrix4::operator -= (const LfMatrix4& m)
{
	r[0] -= m.r[0];
	r[1] -= m.r[1];
	r[2] -= m.r[2];
	r[3] -= m.r[3];

	return *this;
}

// SUBOPTIMAL
inline LfMatrix4& LfMatrix4::operator *= (const LfMatrix4& m)
// WARNING: This routine uses three temporary variables, so, if you need
//          to multiply two matrices two obtain a third matrix, AVOID
//          this method. Use instead:
//            LfMatrix4& Multiply(const LfMatrix4& m1, const LfMatrix4& m2, LfMatrix4& r)
{
	LfFloat t1 = r[0][0];
	LfFloat t2 = r[0][1];
	LfFloat t3 = r[0][2];

	r[0][0] = r[0][0]*_M(m,0,0) + r[0][1]*_M(m,1,0) + r[0][2]*_M(m,2,0) + r[0][3]*_M(m,3,0);
	r[0][1] = t1     *_M(m,0,1) + r[0][1]*_M(m,1,1) + r[0][2]*_M(m,2,1) + r[0][3]*_M(m,3,1);
	r[0][2] = t1     *_M(m,0,2) + t2     *_M(m,1,2) + r[0][2]*_M(m,2,2) + r[0][3]*_M(m,3,2);
	r[0][3] = t1     *_M(m,0,3) + t2     *_M(m,1,3) + t3     *_M(m,2,3) + r[0][3]*_M(m,3,3);


	t1 = r[1][0];
	t2 = r[1][1];
	t3 = r[1][2];

	r[1][0] = r[1][0]*_M(m,0,0) + r[1][1]*_M(m,1,0) + r[1][2]*_M(m,2,0) + r[1][3]*_M(m,3,0);
	r[1][1] = t1     *_M(m,0,1) + r[1][1]*_M(m,1,1) + r[1][2]*_M(m,2,1) + r[1][3]*_M(m,3,1);
	r[1][2] = t1     *_M(m,0,2) + t2     *_M(m,1,2) + r[1][2]*_M(m,2,2) + r[1][3]*_M(m,3,2);
	r[1][3] = t1     *_M(m,0,3) + t2     *_M(m,1,3) + t3     *_M(m,2,3) + r[1][3]*_M(m,3,3);


	t1 = r[2][0];
	t2 = r[2][1];
	t3 = r[2][2];

	r[2][0] = r[2][0]*_M(m,0,0) + r[2][1]*_M(m,1,0) + r[2][2]*_M(m,2,0) + r[2][3]*_M(m,3,0);
	r[2][1] = t1     *_M(m,0,1) + r[2][1]*_M(m,1,1) + r[2][2]*_M(m,2,1) + r[2][3]*_M(m,3,1);
	r[2][2] = t1     *_M(m,0,2) + t2     *_M(m,1,2) + r[2][2]*_M(m,2,2) + r[2][3]*_M(m,3,2);
	r[2][3] = t1     *_M(m,0,3) + t2     *_M(m,1,3) + t3     *_M(m,2,3) + r[2][3]*_M(m,3,3);


	t1 = r[3][0];
	t2 = r[3][1];
	t3 = r[3][2];

	r[3][0] = r[3][0]*_M(m,0,0) + r[3][1]*_M(m,1,0) + r[3][2]*_M(m,2,0) + r[3][3]*_M(m,3,0);
	r[3][1] = t1     *_M(m,0,1) + r[3][1]*_M(m,1,1) + r[3][2]*_M(m,2,1) + r[3][3]*_M(m,3,1);
	r[3][2] = t1     *_M(m,0,2) + t2     *_M(m,1,2) + r[3][2]*_M(m,2,2) + r[3][3]*_M(m,3,2);
	r[3][3] = t1     *_M(m,0,3) + t2     *_M(m,1,3) + t3     *_M(m,2,3) + r[3][3]*_M(m,3,3);

	return *this;
}

inline LfMatrix4& LfMatrix4::operator *= (LfFloat k)
{
	r[0] *= k;
	r[1] *= k;
	r[2] *= k;
	r[3] *= k;

	return *this;
}

inline LfMatrix4& LfMatrix4::operator /= (LfFloat k)
{
	r[0] /= k;
	r[1] /= k;
	r[2] /= k;
	r[3] /= k;

	return *this;
}

inline LfVector4& LfMatrix4::operator [] (int i)
{
	return r[i];
}

inline const LfVector4& LfMatrix4::Get(int i) const
{
	return r[i];
}
inline void LfMatrix4::Set(int i, const LfVector4& v)
{
	r[i] = v;
}

inline LfFloat LfMatrix4::Det() const
{
	return (_CM(*this,0,0)*                                                                              \
			    (_CM(*this,1,1)*(_CM(*this,2,2)*_CM(*this,3,3) - _CM(*this,2,3)*_CM(*this,3,2)) +        \
			     _CM(*this,1,2)*(_CM(*this,2,3)*_CM(*this,3,1) - _CM(*this,2,1)*_CM(*this,3,3)) +        \
			     _CM(*this,1,3)*(_CM(*this,2,1)*_CM(*this,3,2) - _CM(*this,2,2)*_CM(*this,3,1))) +       \
			_CM(*this,0,1)*                                                                              \
			    (_CM(*this,1,0)*(_CM(*this,2,3)*_CM(*this,3,2) - _CM(*this,2,2)*_CM(*this,3,3)) +        \
				 _CM(*this,1,2)*(_CM(*this,2,0)*_CM(*this,3,3) - _CM(*this,2,3)*_CM(*this,3,0)) +        \
				 _CM(*this,1,3)*(_CM(*this,2,2)*_CM(*this,3,0) - _CM(*this,2,0)*_CM(*this,3,2))) +       \
			_CM(*this,0,2)*                                                                              \
			    (_CM(*this,1,0)*(_CM(*this,2,1)*_CM(*this,3,3) - _CM(*this,2,3)*_CM(*this,3,1)) +        \
				 _CM(*this,1,1)*(_CM(*this,2,3)*_CM(*this,3,0) - _CM(*this,2,0)*_CM(*this,3,3)) +        \
				 _CM(*this,1,3)*(_CM(*this,2,0)*_CM(*this,3,1) - _CM(*this,2,1)*_CM(*this,3,0))) +       \
			_CM(*this,0,3)*                                                                              \
			    (_CM(*this,1,0)*(_CM(*this,2,2)*_CM(*this,3,1) - _CM(*this,2,1)*_CM(*this,3,2)) +        \
				 _CM(*this,1,1)*(_CM(*this,2,0)*_CM(*this,3,2) - _CM(*this,2,2)*_CM(*this,3,0)) +        \
				 _CM(*this,1,2)*(_CM(*this,2,1)*_CM(*this,3,0) - _CM(*this,2,0)*_CM(*this,3,1))));       \
}


//
// LfMatrix4 inline functions (not members)
//

inline LfAPI_CS int operator == (const LfMatrix4& a, const LfMatrix4& b)
{
	return ((a.r[0] == b.r[0]) && (a.r[1] == b.r[1]) && (a.r[2] == b.r[2]) && (a.r[3] == b.r[3]));
}

inline LfAPI_CS int operator != (const LfMatrix4& a, const LfMatrix4& b)
{
	return !(a == b);
}

inline LfAPI_CS LfMatrix4 operator  - (const LfMatrix4& a)
{
	return (LfMatrix4(a) *= -1.0);
}

inline LfAPI_CS LfMatrix4 operator  + (const LfMatrix4& a, const LfMatrix4& b)
{
	LfMatrix4 r;
	r.r[0][0] = _M(a,0,0) + _M(b,0,0);
	r.r[0][1] = _M(a,0,1) + _M(b,0,1);
	r.r[0][2] = _M(a,0,2) + _M(b,0,2);
	r.r[0][3] = _M(a,0,3) + _M(b,0,3);
	r.r[1][0] = _M(a,1,0) + _M(b,1,0);
	r.r[1][1] = _M(a,1,1) + _M(b,1,1);
	r.r[1][2] = _M(a,1,2) + _M(b,1,2);
	r.r[1][3] = _M(a,1,3) + _M(b,1,3);
	r.r[2][0] = _M(a,2,0) + _M(b,2,0);
	r.r[2][1] = _M(a,2,1) + _M(b,2,1);
	r.r[2][2] = _M(a,2,2) + _M(b,2,2);
	r.r[2][3] = _M(a,2,3) + _M(b,2,3);
	r.r[3][0] = _M(a,3,0) + _M(b,3,0);
	r.r[3][1] = _M(a,3,1) + _M(b,3,1);
	r.r[3][2] = _M(a,3,2) + _M(b,3,2);
	r.r[3][3] = _M(a,3,3) + _M(b,3,3);
	return r;
}

inline LfAPI_CS LfMatrix4 operator  - (const LfMatrix4& a, const LfMatrix4& b)
{
	LfMatrix4 r;
	r.r[0][0] = _M(a,0,0) - _M(b,0,0);
	r.r[0][1] = _M(a,0,1) - _M(b,0,1);
	r.r[0][2] = _M(a,0,2) - _M(b,0,2);
	r.r[0][3] = _M(a,0,3) - _M(b,0,3);
	r.r[1][0] = _M(a,1,0) - _M(b,1,0);
	r.r[1][1] = _M(a,1,1) - _M(b,1,1);
	r.r[1][2] = _M(a,1,2) - _M(b,1,2);
	r.r[1][3] = _M(a,1,3) - _M(b,1,3);
	r.r[2][0] = _M(a,2,0) - _M(b,2,0);
	r.r[2][1] = _M(a,2,1) - _M(b,2,1);
	r.r[2][2] = _M(a,2,2) - _M(b,2,2);
	r.r[2][3] = _M(a,2,3) - _M(b,2,3);
	r.r[3][0] = _M(a,3,0) - _M(b,3,0);
	r.r[3][1] = _M(a,3,1) - _M(b,3,1);
	r.r[3][2] = _M(a,3,2) - _M(b,3,2);
	r.r[3][3] = _M(a,3,3) - _M(b,3,3);
	return r;
}

inline LfAPI_CS LfMatrix4 operator  * (const LfMatrix4& a, const LfMatrix4& b)
{
	LfMatrix4 r;

	return Multiply(a, b, r);
}

inline LfAPI_CS LfMatrix4 operator  * (const LfMatrix4& a, LfFloat k)
{
	LfMatrix4 r;
	r.r[0][0] = _M(a,0,0) * k;
	r.r[0][1] = _M(a,0,1) * k;
	r.r[0][2] = _M(a,0,2) * k;
	r.r[0][3] = _M(a,0,3) * k;
	r.r[1][0] = _M(a,1,0) * k;
	r.r[1][1] = _M(a,1,1) * k;
	r.r[1][2] = _M(a,1,2) * k;
	r.r[1][3] = _M(a,1,3) * k;
	r.r[2][0] = _M(a,2,0) * k;
	r.r[2][1] = _M(a,2,1) * k;
	r.r[2][2] = _M(a,2,2) * k;
	r.r[2][3] = _M(a,2,3) * k;
	r.r[3][0] = _M(a,3,0) * k;
	r.r[3][1] = _M(a,3,1) * k;
	r.r[3][2] = _M(a,3,2) * k;
	r.r[3][3] = _M(a,3,3) * k;
	return r;
}

inline LfAPI_CS LfMatrix4 operator  * (LfFloat k, const LfMatrix4& a)
{
	return (a * k);
}

inline LfAPI_CS LfVector4 operator * (const LfVector4& v, const LfMatrix4& m)
{
	LfVector4 r;

	return Multiply(v, m, r);
}

inline LfAPI_CS LfVector4 operator * (const LfMatrix4& m, const LfVector4& v)
{
	LfVector4 r;

	return Multiply(m, v, r);
}

inline LfAPI_CS LfMatrix4& Multiply(const LfMatrix4& a, const LfMatrix4& b, LfMatrix4& r)
{
	r.r[0][0] = _M(a,0,0)*_M(b,0,0) + _M(a,0,1)*_M(b,1,0) + _M(a,0,2)*_M(b,2,0) + _M(a,0,3)*_M(b,3,0);
	r.r[0][1] = _M(a,0,0)*_M(b,0,1) + _M(a,0,1)*_M(b,1,1) + _M(a,0,2)*_M(b,2,1) + _M(a,0,3)*_M(b,3,1);
	r.r[0][2] = _M(a,0,0)*_M(b,0,2) + _M(a,0,1)*_M(b,1,2) + _M(a,0,2)*_M(b,2,2) + _M(a,0,3)*_M(b,3,2);
	r.r[0][3] = _M(a,0,0)*_M(b,0,3) + _M(a,0,1)*_M(b,1,3) + _M(a,0,2)*_M(b,2,3) + _M(a,0,3)*_M(b,3,3);

	r.r[1][0] = _M(a,1,0)*_M(b,0,0) + _M(a,1,1)*_M(b,1,0) + _M(a,1,2)*_M(b,2,0) + _M(a,1,3)*_M(b,3,0);
	r.r[1][1] = _M(a,1,0)*_M(b,0,1) + _M(a,1,1)*_M(b,1,1) + _M(a,1,2)*_M(b,2,1) + _M(a,1,3)*_M(b,3,1);
	r.r[1][2] = _M(a,1,0)*_M(b,0,2) + _M(a,1,1)*_M(b,1,2) + _M(a,1,2)*_M(b,2,2) + _M(a,1,3)*_M(b,3,2);
	r.r[1][3] = _M(a,1,0)*_M(b,0,3) + _M(a,1,1)*_M(b,1,3) + _M(a,1,2)*_M(b,2,3) + _M(a,1,3)*_M(b,3,3);

	r.r[2][0] = _M(a,2,0)*_M(b,0,0) + _M(a,2,1)*_M(b,1,0) + _M(a,2,2)*_M(b,2,0) + _M(a,2,3)*_M(b,3,0);
	r.r[2][1] = _M(a,2,0)*_M(b,0,1) + _M(a,2,1)*_M(b,1,1) + _M(a,2,2)*_M(b,2,1) + _M(a,2,3)*_M(b,3,1);
	r.r[2][2] = _M(a,2,0)*_M(b,0,2) + _M(a,2,1)*_M(b,1,2) + _M(a,2,2)*_M(b,2,2) + _M(a,2,3)*_M(b,3,2);
	r.r[2][3] = _M(a,2,0)*_M(b,0,3) + _M(a,2,1)*_M(b,1,3) + _M(a,2,2)*_M(b,2,3) + _M(a,2,3)*_M(b,3,3);

	r.r[3][0] = _M(a,3,0)*_M(b,0,0) + _M(a,3,1)*_M(b,1,0) + _M(a,3,2)*_M(b,2,0) + _M(a,3,3)*_M(b,3,0);
	r.r[3][1] = _M(a,3,0)*_M(b,0,1) + _M(a,3,1)*_M(b,1,1) + _M(a,3,2)*_M(b,2,1) + _M(a,3,3)*_M(b,3,1);
	r.r[3][2] = _M(a,3,0)*_M(b,0,2) + _M(a,3,1)*_M(b,1,2) + _M(a,3,2)*_M(b,2,2) + _M(a,3,3)*_M(b,3,2);
	r.r[3][3] = _M(a,3,0)*_M(b,0,3) + _M(a,3,1)*_M(b,1,3) + _M(a,3,2)*_M(b,2,3) + _M(a,3,3)*_M(b,3,3);

	return r;
}

// OPTIMAL
inline LfAPI_CS LfVector4& Multiply(const LfVector4& v, const LfMatrix4& m, LfVector4& r)
{
	r[0] = _V(v,0)*_M(m,0,0) + _V(v,1)*_M(m,1,0) +
	       _V(v,2)*_M(m,2,0) + _V(v,3)*_M(m,3,0);
	r[1] = _V(v,0)*_M(m,0,1) + _V(v,1)*_M(m,1,1) +
	       _V(v,2)*_M(m,2,1) + _V(v,3)*_M(m,3,1);
	r[2] = _V(v,0)*_M(m,0,2) + _V(v,1)*_M(m,1,2) +
	       _V(v,2)*_M(m,2,2) + _V(v,3)*_M(m,3,2);
	r[3] = _V(v,0)*_M(m,0,3) + _V(v,1)*_M(m,1,3) +
	       _V(v,2)*_M(m,2,3) + _V(v,3)*_M(m,3,3);

	return r;
}

// OPTIMAL
inline LfAPI_CS LfVector4& Multiply(const LfMatrix4& m, const LfVector4& v, LfVector4& r)
{
	r[0] = _M(m,0,0)*_V(v,0) + _M(m,0,1)*_V(v,1) +
	       _M(m,0,2)*_V(v,2) + _M(m,0,3)*_V(v,3);
	r[1] = _M(m,1,0)*_V(v,0) + _M(m,1,1)*_V(v,1) +
           _M(m,1,2)*_V(v,2) + _M(m,1,3)*_V(v,3);
	r[2] = _M(m,2,0)*_V(v,0) + _M(m,2,1)*_V(v,1) +
	       _M(m,2,2)*_V(v,2) + _M(m,2,3)*_V(v,3);
	r[3] = _M(m,3,0)*_V(v,0) + _M(m,3,1)*_V(v,1) +
	       _M(m,3,2)*_V(v,2) + _M(m,3,3)*_V(v,3);

	return r;
}

inline LfAPI_CS LfMatrix4 operator  / (const LfMatrix4& a, LfFloat k)
{
	LfMatrix4 r;
	r.r[0][0] = _M(a,0,0) / k;
	r.r[0][1] = _M(a,0,1) / k;
	r.r[0][2] = _M(a,0,2) / k;
	r.r[0][3] = _M(a,0,3) / k;
	r.r[1][0] = _M(a,1,0) / k;
	r.r[1][1] = _M(a,1,1) / k;
	r.r[1][2] = _M(a,1,2) / k;
	r.r[1][3] = _M(a,1,3) / k;
	r.r[2][0] = _M(a,2,0) / k;
	r.r[2][1] = _M(a,2,1) / k;
	r.r[2][2] = _M(a,2,2) / k;
	r.r[2][3] = _M(a,2,3) / k;
	r.r[3][0] = _M(a,3,0) / k;
	r.r[3][1] = _M(a,3,1) / k;
	r.r[3][2] = _M(a,3,2) / k;
	r.r[3][3] = _M(a,3,3) / k;

	return r;
}

inline LfAPI_CS LfMatrix4 Transpose(const LfMatrix4& m)
{
	LfMatrix4 r;

	return Transpose(m, r);
}

inline LfAPI_CS LfMatrix4& Transpose(const LfMatrix4& m, LfMatrix4& r)
{
	r.r[0][0] = _M(m,0,0);
	r.r[0][1] = _M(m,1,0);
	r.r[0][2] = _M(m,2,0);
	r.r[0][3] = _M(m,3,0);
	r.r[1][0] = _M(m,0,1);
	r.r[1][1] = _M(m,1,1);
	r.r[1][2] = _M(m,2,1);
	r.r[1][3] = _M(m,3,1);
	r.r[2][0] = _M(m,0,2);
	r.r[2][1] = _M(m,1,2);
	r.r[2][2] = _M(m,2,2);
	r.r[2][3] = _M(m,3,2);
	r.r[3][0] = _M(m,0,3);
	r.r[3][1] = _M(m,1,3);
	r.r[3][2] = _M(m,2,3);
	r.r[3][3] = _M(m,3,3);

	return r;
}

inline LfAPI_CS int Invert(const LfMatrix4& a, LfMatrix4& r)
{
	LfFloat t1, t2, t3, t4, t5, t6;

	// We calculate the adjoint matrix of a, then we divide it by the determinant of a
	// We indicate with Aij the cofactor of a[i][j]:   Aij = (-1)^(i+j)*Det(Tij), where
	// Tij is the minor complementary of a[i][j]

	// First block ([0,0] - [3,1])

	t1 = _M(a,2,2)*_M(a,3,3) - _M(a,2,3)*_M(a,3,2);
	t2 = _M(a,2,1)*_M(a,3,3) - _M(a,2,3)*_M(a,3,1);
	t3 = _M(a,2,1)*_M(a,3,2) - _M(a,2,2)*_M(a,3,1);
	t4 = _M(a,2,0)*_M(a,3,3) - _M(a,2,3)*_M(a,3,0);
	t5 = _M(a,2,0)*_M(a,3,2) - _M(a,2,2)*_M(a,3,0);
	t6 = _M(a,2,0)*_M(a,3,1) - _M(a,2,1)*_M(a,3,0);

	r.r[0][0] =   _M(a,1,1) * t1 - _M(a,1,2) * t2 + _M(a,1,3) * t3;      // A00
	r.r[0][1] = -(_M(a,0,1) * t1 - _M(a,0,2) * t2 + _M(a,0,3) * t3);     // A10
	r.r[1][0] = -(_M(a,1,0) * t1 - _M(a,1,2) * t4 + _M(a,1,3) * t5);     // A01
	r.r[1][1] =   _M(a,0,0) * t1 - _M(a,0,2) * t4 + _M(a,0,3) * t5;      // A11
	r.r[2][0] =   _M(a,1,0) * t2 - _M(a,1,1) * t4 + _M(a,1,3) * t6;      // A02
	r.r[2][1] = -(_M(a,0,0) * t2 - _M(a,0,1) * t4 + _M(a,0,3) * t6);     // A21
	r.r[3][0] = -(_M(a,1,0) * t3 - _M(a,1,1) * t5 + _M(a,1,2) * t6);     // A03
	r.r[3][1] =   _M(a,0,0) * t3 - _M(a,0,1) * t5 + _M(a,0,2) * t6;      // A13

	// Second block ([0,2] - [3,2])

	t1 = _M(a,1,2)*_M(a,3,3) - _M(a,1,3)*_M(a,3,2);
	t2 = _M(a,1,1)*_M(a,3,3) - _M(a,1,3)*_M(a,3,1);
	t3 = _M(a,1,1)*_M(a,3,2) - _M(a,1,2)*_M(a,3,1);
	t4 = _M(a,1,0)*_M(a,3,3) - _M(a,1,3)*_M(a,3,0);
	t5 = _M(a,1,0)*_M(a,3,2) - _M(a,1,2)*_M(a,3,0);
	t6 = _M(a,1,0)*_M(a,3,1) - _M(a,1,1)*_M(a,3,0);

	r.r[0][2] =   _M(a,0,1) * t1 - _M(a,0,2) * t2 + _M(a,0,3) * t3;      // A20
	r.r[1][2] = -(_M(a,0,0) * t1 - _M(a,0,2) * t4 + _M(a,0,3) * t5);     // A21
	r.r[2][2] =   _M(a,0,0) * t2 - _M(a,0,1) * t4 + _M(a,0,3) * t6;      // A22
	r.r[3][2] = -(_M(a,0,0) * t3 - _M(a,0,1) * t5 + _M(a,0,2) * t6);     // A23

	// Third block ([0,3] - [3,3])

	t1 = _M(a,1,2)*_M(a,2,3) - _M(a,1,3)*_M(a,2,2);
	t2 = _M(a,1,1)*_M(a,2,3) - _M(a,1,3)*_M(a,2,1);
	t3 = _M(a,1,1)*_M(a,2,2) - _M(a,1,2)*_M(a,2,1);
	t4 = _M(a,1,0)*_M(a,2,3) - _M(a,1,3)*_M(a,2,0);
	t5 = _M(a,1,0)*_M(a,2,2) - _M(a,1,2)*_M(a,2,0);
	t6 = _M(a,1,0)*_M(a,2,1) - _M(a,1,1)*_M(a,2,0);

	r.r[0][3] = -(_M(a,0,1) * t1 - _M(a,0,2) * t2 + _M(a,0,3) * t3);     // A30
	r.r[1][3] =   _M(a,0,0) * t1 - _M(a,0,2) * t4 + _M(a,0,3) * t5;      // A31
	r.r[2][3] = -(_M(a,0,0) * t2 - _M(a,0,1) * t4 + _M(a,0,3) * t6);     // A32
	r.r[3][3] =   _M(a,0,0) * t3 - _M(a,0,1) * t5 + _M(a,0,2) * t6;      // A33

	// We save some time calculating Det(a) this way (now r is adjoint of a)
	// Det(a) = a00 * A00 + a01 * A01 + a02 * A02 + a03 * A03
	LfFloat d = _M(a,0,0)*r.r[0][0] + _M(a,0,1)*r.r[1][0] + _M(a,0,2)*r.r[2][0] + _M(a,0,3)*r.r[3][0];

	if (__LfZero__(d)) return 0; // Singular matrix => no inverse

	d = 1/d;

	r *= d;

	return 1;
}

inline LfAPI_CS LfFloat Det(const LfMatrix4& m)
{
	return m.Det();
}

#undef _M
#undef _CM

#endif /* _LfMatrix4_h */
